/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for TI TM4C129 Development Kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "HW_TM4C129.h"
#include "rom.h"
#include "RTOS.h"
#include "TM4C129.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define RCGCGPIO            (*(volatile unsigned int*)(0x400FE608))

#define GPION_BASE_ADDR     (0x40064000u)
#define GPION_GPIODEN       (*(volatile unsigned int*)(GPION_BASE_ADDR + 0x51Cu))
#define GPION_GPIODIR       (*(volatile unsigned int*)(GPION_BASE_ADDR + 0x400u))
#define GPION5_GPIODATA     (*(volatile unsigned int*)(GPION_BASE_ADDR + 0x080u))
#define RCGCGPIO_GPION_BIT  (12u)

#define GPIOQ_BASE_ADDR     (0x40066000)
#define GPIOQ_GPIODEN       (*(volatile unsigned int*)(GPIOQ_BASE_ADDR + 0x51Cu))
#define GPIOQ_GPIODIR       (*(volatile unsigned int*)(GPIOQ_BASE_ADDR + 0x400u))
#define GPIOQ4_GPIODATA     (*(volatile unsigned int*)(GPIOQ_BASE_ADDR + 0x040u))
#define GPIOQ7_GPIODATA     (*(volatile unsigned int*)(GPIOQ_BASE_ADDR + 0x200u))
#define RCGCGPIO_GPIOQ_BIT  (14u)

#define LED0                (5u)  // D12, PN5/LED_R
#define LED1                (4u)  // D12, PQ4/LED_B
#define LED2                (7u)  // D12, PQ7/LED_G

#define GPIO_LOCK_KEY       0x4C4F434B  // Unlocks the GPIO_CR register

#define SYSCTL_RCC2             0x400FE070  // Run-Mode Clock Configuration 2
#define SYSCTL_RCC2_USBPWRDN    0x00004000  // Power-Down USB PLL

#define USB_O_CC                0x00000FC8  // USB Clock Configuration

/*********************************************************************
*
*       Typedefs
*
**********************************************************************
*/

typedef void USB_ISR_HANDLER  (void);

/*********************************************************************
*
*       Static data
*
**********************************************************************
*/

static USB_ISR_HANDLER * _pfUSBISRHandler;

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       USB
*
*  Functions for USB controllers (as far as present)
*/
/****** Declare ISR handler here to avoid "no prototype" warning. They are not declared in any CMSIS header */
#ifdef __cplusplus
extern "C" {
#endif
void USB0_Handler(void);
#ifdef __cplusplus
}
#endif

/*********************************************************************
*
*       USB0_Handler
*/
void USB0_Handler(void) {
  OS_EnterInterrupt(); // Inform embOS that interrupt code is running
  if (_pfUSBISRHandler) {
    (_pfUSBISRHandler)();
  }
  OS_LeaveInterrupt(); // Inform embOS that interrupt code is left
}



/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  RCGCGPIO |= (1u << RCGCGPIO_GPION_BIT); // Enable power for port N
  RCGCGPIO |= (1u << RCGCGPIO_GPIOQ_BIT); // Enable power for port Q

  GPION_GPIODEN   |=  (1u << LED0);       // Enable pad
  GPION_GPIODIR   |= (1u << LED0);        // Set to output
  GPION5_GPIODATA &= ~(1u << LED0);       // Initially clear LED

  GPIOQ_GPIODEN   |= (1u << LED1);        // Enable pad
  GPIOQ_GPIODIR   |= (1u << LED1);        // Set to output
  GPIOQ4_GPIODATA &= ~(1u << LED1);       // Initially clear LED

  GPIOQ_GPIODEN   |= (1u << LED2);        // Enable pad
  GPIOQ_GPIODIR   |= (1u << LED2);        // Set to output
  GPIOQ7_GPIODATA &= ~(1u << LED2);       // Initially clear LED
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
  GPION5_GPIODATA |= (1u << LED0);
  } else if (Index == 1) {
    GPIOQ4_GPIODATA |= (1u << LED1);
  } else if (Index == 2) {
     GPIOQ7_GPIODATA |= (1u << LED2);
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    GPION5_GPIODATA &= ~(1u << LED0);
  } else if (Index == 1) {
    GPIOQ4_GPIODATA &= ~(1u << LED1);
  } else if (Index == 2) {
    GPIOQ7_GPIODATA &= ~(1u << LED2);
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if (GPION5_GPIODATA != 0) {
      GPION5_GPIODATA &= ~(1u << LED0);
    } else {
      GPION5_GPIODATA |= (1u << LED0);
    }
  } else if (Index == 1) {
    if (GPIOQ4_GPIODATA != 0) {
      GPIOQ4_GPIODATA &= ~(1u << LED1);
  } else {
      GPIOQ4_GPIODATA |= (1u << LED1);
  }
  } else if (Index == 2) {
    if (GPIOQ7_GPIODATA != 0) {
    GPIOQ7_GPIODATA &= ~(1u << LED2);
    } else {
      GPIOQ7_GPIODATA |= (1u << LED2);
    }
  }
}

void BSP_USB_Init(void) {
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOB);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOD);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOL);
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_USB0);
  //
  // PB0-1/PD6-7/PL6-7 are used for USB.
  //
  HWREG(GPIO_PORTD_BASE + GPIO_O_LOCK) = GPIO_LOCK_KEY;
  HWREG(GPIO_PORTD_BASE + GPIO_O_CR) = 0xff;
  ROM_GPIOPinConfigure(GPIO_PD6_USB0EPEN);
  ROM_GPIOPinConfigure(GPIO_PD7_USB0PFLT);
  ROM_GPIOPinTypeUSBAnalog(GPIO_PORTB_BASE, GPIO_PIN_0 | GPIO_PIN_1);
  ROM_GPIOPinTypeUSBDigital(GPIO_PORTD_BASE, GPIO_PIN_6 | GPIO_PIN_7);
  ROM_GPIOPinTypeUSBAnalog(GPIO_PORTL_BASE, GPIO_PIN_6 | GPIO_PIN_7);
  HWREGBITW(SYSCTL_RCGCBASE + ((SYSCTL_PERIPH_USB0 & 0xff00) >> 8), SYSCTL_PERIPH_USB0 & 0xff) = 1;
  //
  // Reset the USB controller.
  //
  ROM_SysCtlPeripheralReset(SYSCTL_PERIPH_USB0);
  //
  // Enable Clocking to the USB controller.
  //
  ROM_SysCtlPeripheralEnable(SYSCTL_PERIPH_USB0);
  //
  // Turn on USB Phy clock.
  //
  HWREG(SYSCTL_RCC2) &= ~SYSCTL_RCC2_USBPWRDN;
  //
  // Configure and enable the USB clock input.
  //
  HWREG(USB0_BASE + USB_O_CC) = (8 - 1) | 0x200;
  OS_Delay(100);
}

/*********************************************************************
*
*       BSP_USB_InstallISR
*/
void BSP_USB_InstallISR(void (*pfISR)(void)) {
  _pfUSBISRHandler = pfISR;
  NVIC_SetPriority(USB0_IRQn, (1u << __NVIC_PRIO_BITS) - 2u);
  NVIC_EnableIRQ(USB0_IRQn);

}


/****** End Of File *************************************************/
